<?php
/**
 * @package     Rovexo.Plugin
 * @subpackage  System.jrequire
 *
 * @copyright   Copyright (C) 2018 Rovexo. All rights reserved.
 * @license		GNU General Public License version 2 or later OR MIT
 */
defined('_JEXEC') or die;

/**
 * Plugin class.
 *
 * @since  1.0
 */
class PlgSystemJrequire extends JPlugin {
	/**
	 *
	 * @return  void
	 *
	 * @since   1.0
	 */
	public function onBeforeRender() {

		$db = JFactory::getDbo();
		$db->setQuery("SELECT enabled FROM #__extensions WHERE element = 'lib_jrequire'");
		$enabled = $db->loadResult();

		if ($enabled != '1') {
			return;
		}

		// Get the base URL of the installation
		$urlBase = JUri::getInstance()->base();

		$urlBase = rtrim($urlBase, '/');

		// On admin, remove the /administrator
		if (JFactory::getApplication()->isClient('administrator')) {
			// Just in case Joomla ever gives base() without /administrator, check if it's actually there
			$pos = strrpos($urlBase, '/administrator');
			if ($pos !== false) {
				$urlBase = mb_substr($urlBase, 0, $pos);
			}
		}

		$debug = JFactory::getApplication()->get('debug');

		if ($debug) {
			$urlRequireJs = $urlBase.'/media/plg_system_jrequire/js/require-2.3.5/require.js';
			$urlMainJs = $urlBase.'/media/plg_system_jrequire/js/main.js';
		}
		else {
			$urlRequireJs = $urlBase.'/media/plg_system_jrequire/js/require-2.3.5/require.min.js';
			$urlMainJs = $urlBase.'/media/plg_system_jrequire/js/main.min.js';
		}

		jimport('jrequire.JRequire');

		JRequire::addModuleCall('media/sample/something', 'initCall');

		// We put a whole lot of settings as JSON string in a data attribute of the requireJS script tag
		// It will be read and stored in the main.js file and ready to use via the configbox JS module (configbox.config)
		$appConfig = array(
			'urlBase'           => $urlBase,
			'calls'				=> JRequire::getModuleCalls(),
		);

		// You can create a function called 'cbGetCustomRequirePaths' to add paths to the requireJS configuration
		$appConfig['customPaths'] = JRequire::getCustomPaths();

		// You can create a function called 'cbGetCustomRequireShims' to add shims to the requireJS configuration
		$appConfig['customShims'] = JRequire::getCustomShims();

		// Prepare JS that will add our requireJS script tag to the HTML docs head (including our settings JS)
		$js = "
		(function (urlRequireJs, urlMainJs, appConfig) {
			
			// Let any other JS load first to avoid non-AMD-style loaded JS that uses define (avoiding anon mod mismatches)
			window.addEventListener('load', function() {
							
				// In case requireJS is already loaded by another, then just add our own main.js file
				if (typeof(require) !== 'undefined') {
					var mainScript = document.createElement('script');
					mainScript.id = 'jrequire-main-file-tag';
					mainScript.async = 1;
					mainScript.src = urlMainJs;
					mainScript.dataset.appConfig = appConfig;
					document.getElementsByTagName('head')[0].appendChild(mainScript);
				}
				else {
					var requireJsScript = document.createElement('script');
					requireJsScript.id = 'jrequire-require-tag';
					requireJsScript.async = 1;
					requireJsScript.src = urlRequireJs;
					requireJsScript.dataset.main = urlMainJs;
					requireJsScript.dataset.appConfig = appConfig;
					document.getElementsByTagName('head')[0].appendChild(requireJsScript);
				}
				
			});
							
		})('".$urlRequireJs."', '".$urlMainJs."', '".json_encode($appConfig)."');
		";

		JFactory::getDocument()->addScriptDeclaration($js);

	}

}
